# Use Case: Non-trainable Model

## Introduction

The simplest case to deal with, is one where the model is a non-learning algorithm. 

This case is (and should be) the most common that you encounter. Your entire Business Intelligence unit is delivering models that are mostly deterministic, heavily relying on business logic.

## Getting Started

The first action to take is to create the layout for your model.

First, make sure that `blackboxops` is installed in your system.
```
pip install blackboxops --upgrade
```
We also use `poetry` as our default package manager. 
```
pip install poetry --upgrade
```
You can find more information about `poetry` at https://python-poetry.org/.



Then, let's create our working layout 
```
python -m blackboxops --name ProjectName --noflow
```
Here, the flag `--noflow` ensures that we only generate the model baseline, without any trainable functionalities. The output of this action will be a structure
```
├── ProjectName
│   ├── architecture.py
│   ├── __init__.py
│   ├── bookkeeper
│   │   └── __init__.py
│   ├── model
│   │   └── __init__.py
│   └── performance
│       └── __init__.py
├── pyproject.toml
```
Let's make sure that everything has been installed correctly by issuing
```
poetry install
```

## More Poetry

If your model will be deployed directly inside another Python project (not a great workflow, but still a valid one), then make sure to copy/paste the dependencies of that project into your `pyproject.toml` file before you continue. This will ensure that your wheel file will be properly deployed into your main project.

## Definining Version and Performance

In the file `ProjectName/architecture.py` you will redefine `version` and `performance` as follow

```
def version():
    return(
        'X.y' # X = Main, y = minor
    )
```
according to how you'd like to version this model. For the entirity of its lifetime, the model will have a version of the form `X.y.0000`. It is your responsibility to manage the major and minor numbers of the version.

Because the performance object won't be doing any calculations in the background, you can simply definte it as
```
def backend_performance():
    return(
        Performance() # Default Performance Object
    )
```
The default performance object doesn't do anything interesting (in this case, it doesn't have to)

## Definining Your Model
Now we get to the fun part. Let's assume that we are interested in building a model that receives a list of numbers and can return its sum and average. To make this a bit less trivial, we'll assume that we're forced to use Numpy (in the command line, execute `poetry add numpy`).

Let's first define the model
```
import numpy as np

def backend_model():
    return(None) # No backend model needed
```
By defining its backend as `None` you're explicitely assuming that this model doesn't need to keep track of anything in the backend.

Let's get down to the functionalities

```
# Define functionalities
# -- SUM
def compute_sum(x_list):
    return(np.sum(x_list))

#- -- MEAN
def compute_mean(x_list):
    return(np.mean(x_list))
```
Finally, we register these functionalities via the method `interface_methods` which packs them into a dictionary of methods to be used by our model interface, via the `execute` method (more about this later).

```
# Design Methods for Interface
# -- register interface methods
def interface_methods():
    ret = {}
    # -- SUM
    def sum(backend_mdl, x_list):
        return(
            compute_sum(x_list)
        )
    ret['sum'] = sum

    #- -- MEAN
    def mean(backend_mdl, x_list):
        return(
            compute_mean(x_list)
        )
    ret['mean'] = mean

    return(ret)
```
At this point, we have defined our functionalities (`compute_sum` and `compute_mean`) and attach them to our model's interface.

If everything is going well, after firing
```
poetry build
```
You'd have a ready to deploy model at
```
dist/ProjectName-0.1.0-py3-none-any.whl
```

## Using Your Model

After you have deployed your model, for examply, by
* loading `dist/ProjectName-0.1.0-py3-none-any.whl` to pypi.org; or
* copying `dist/ProjectName-0.1.0-py3-none-any.whl` directly inside a project that allows you to do `pip install path-to/ProjectName-0.1.0-py3-none-any.whl`;
* etc.

you'd be in position to use your model. Here it's how you can use it

```
# Some script using ProjectName model
import ProjectName

mdl = ProjectName.Model()

print(mdl)

my_list = [1, 2, 3, 4, 5, 6]
print("my_list = {}".format(my_list))

my_sum = mdl.execute('sum', my_list)
print("my_sum = {}".format(my_sum))

my_mean = mdl.execute('mean', my_list)
print("my_mean = {}".format(my_mean))
```
You should see something like
```
<ProjectName.model.Model object at 0x10e088250>
my_list = [1, 2, 3, 4, 5, 6]
my_sum = 21
my_mean = 3.5
```

## Testing
Before you're ready to commit your model, include some testing. In this case, we would like to make sure that the interface we're defining is never broken. 
In `tests/test_methods.py` file, add the following tests (for this simple example, one per defined method).

```
from ProjectName import Model

mdl = Model()

my_list = [1, 2, 3, 4, 5, 6]

def test_sum():
    my_sum = 21
    assert my_sum == mdl.execute('sum', my_list)

def test_mean():
    my_mean = 3.5
    assert my_mean == mdl.execute('mean', my_list)
```
and run (in your command line)
```
poetry run pytest
```
If everything is going well, your tests should have passed.

**Congratulations, you have created your first (non-trainable) model**
