import pytest
from helloml.performance import Performance

import numpy as np
import sklearn.metrics

VERSION_LENGTH_LIST = [1, 5, 10, 15]
RANDOM_SIZE_LIST = [10, 50, 100]

PERF_PARAMS_LIST = [{
    'metric_function': sklearn.metrics.mean_squared_error,
    'abreviation': 'mse',
    'min_value': 0.0,
    'max_value': float('Inf'),
    'is_bigger_better': False,
},
    {
    'metric_function': sklearn.metrics.mean_absolute_error,
    'abreviation': 'mae',
    'min_value': 0.0,
    'max_value': float('Inf'),
    'is_bigger_better': False,
}]


@pytest.mark.parametrize("VERSION_LENGTH", VERSION_LENGTH_LIST)
@pytest.mark.parametrize("PERF_PARAMS", PERF_PARAMS_LIST)
def test_default_value(PERF_PARAMS, VERSION_LENGTH):
    perf = Performance(metric_function=PERF_PARAMS['metric_function'],
                       abreviation=PERF_PARAMS['abreviation'],
                       min_value=PERF_PARAMS['min_value'],
                       max_value=PERF_PARAMS['max_value'],
                       is_bigger_better=PERF_PARAMS['is_bigger_better'],
                       version_length=VERSION_LENGTH)
    assert perf.current_value is not None


@pytest.mark.parametrize("VERSION_LENGTH", VERSION_LENGTH_LIST)
@pytest.mark.parametrize("PERF_PARAMS", PERF_PARAMS_LIST)
def test_default_version(PERF_PARAMS, VERSION_LENGTH):
    perf = Performance(metric_function=PERF_PARAMS['metric_function'],
                       abreviation=PERF_PARAMS['abreviation'],
                       min_value=PERF_PARAMS['min_value'],
                       max_value=PERF_PARAMS['max_value'],
                       is_bigger_better=PERF_PARAMS['is_bigger_better'],
                       version_length=VERSION_LENGTH)

    target_version = 0.0
    assert perf.version == str(int(target_version * (10**VERSION_LENGTH)))


@pytest.mark.parametrize("VERSION_LENGTH", VERSION_LENGTH_LIST)
@pytest.mark.parametrize("PERF_PARAMS", PERF_PARAMS_LIST)
@pytest.mark.parametrize("RANDOM_SIZE", RANDOM_SIZE_LIST)
def test_set_version_one(PERF_PARAMS, VERSION_LENGTH, RANDOM_SIZE):
    perf = Performance(metric_function=PERF_PARAMS['metric_function'],
                       abreviation=PERF_PARAMS['abreviation'],
                       min_value=PERF_PARAMS['min_value'],
                       max_value=PERF_PARAMS['max_value'],
                       is_bigger_better=PERF_PARAMS['is_bigger_better'],
                       version_length=VERSION_LENGTH)
    y_true = np.random.normal(size=(RANDOM_SIZE, 1))
    y_pred = y_true
    perf.set_current_value(y_true=y_true, y_pred=y_pred)

    target_version = 1.0 - 1.0 / (10**VERSION_LENGTH)
    assert perf.version == str(int(target_version * (10**VERSION_LENGTH)))


@pytest.mark.parametrize("VERSION_LENGTH", VERSION_LENGTH_LIST)
@pytest.mark.parametrize("PERF_PARAMS", PERF_PARAMS_LIST)
@pytest.mark.parametrize("RANDOM_SIZE", RANDOM_SIZE_LIST)
def test_set_current_zero(PERF_PARAMS, VERSION_LENGTH, RANDOM_SIZE):
    perf = Performance(metric_function=PERF_PARAMS['metric_function'],
                       abreviation=PERF_PARAMS['abreviation'],
                       min_value=PERF_PARAMS['min_value'],
                       max_value=PERF_PARAMS['max_value'],
                       is_bigger_better=PERF_PARAMS['is_bigger_better'],
                       version_length=VERSION_LENGTH)

    y_true = np.random.normal(size=(RANDOM_SIZE, 1))
    y_pred = y_true
    perf.set_current_value(y_true=y_true, y_pred=y_pred)
    assert perf.current_value == 0.0


@pytest.mark.parametrize("VERSION_LENGTH", VERSION_LENGTH_LIST)
@pytest.mark.parametrize("PERF_PARAMS", PERF_PARAMS_LIST)
@pytest.mark.parametrize("RANDOM_SIZE", RANDOM_SIZE_LIST)
def test_equal_operator(PERF_PARAMS, VERSION_LENGTH, RANDOM_SIZE):
    perf_1 = Performance(metric_function=PERF_PARAMS['metric_function'],
                         abreviation=PERF_PARAMS['abreviation'],
                         min_value=PERF_PARAMS['min_value'],
                         max_value=PERF_PARAMS['max_value'],
                         is_bigger_better=PERF_PARAMS['is_bigger_better'],
                         version_length=VERSION_LENGTH)
    ADD = 1.00
    y_true = np.random.normal(size=(RANDOM_SIZE, 1))
    y_pred = y_true + np.ones(shape=y_true.shape)*ADD

    perf_1.set_current_value(y_true=y_true, y_pred=y_pred)

    perf_2 = Performance(metric_function=PERF_PARAMS['metric_function'],
                         abreviation=PERF_PARAMS['abreviation'],
                         min_value=PERF_PARAMS['min_value'],
                         max_value=PERF_PARAMS['max_value'],
                         is_bigger_better=PERF_PARAMS['is_bigger_better'],
                         version_length=VERSION_LENGTH)

    perf_2.set_current_value(y_true=y_true, y_pred=y_pred)

    assert (perf_1 == perf_2)


@pytest.mark.parametrize("VERSION_LENGTH", VERSION_LENGTH_LIST)
@pytest.mark.parametrize("PERF_PARAMS", PERF_PARAMS_LIST)
@pytest.mark.parametrize("RANDOM_SIZE", RANDOM_SIZE_LIST)
def test_noteq_operator(PERF_PARAMS, VERSION_LENGTH, RANDOM_SIZE):
    perf_1 = Performance(metric_function=PERF_PARAMS['metric_function'],
                         abreviation=PERF_PARAMS['abreviation'],
                         min_value=PERF_PARAMS['min_value'],
                         max_value=PERF_PARAMS['max_value'],
                         is_bigger_better=PERF_PARAMS['is_bigger_better'],
                         version_length=VERSION_LENGTH)
    ADD = 1.00
    y_true = np.random.normal(size=(RANDOM_SIZE, 1))
    y_pred = y_true + np.ones(shape=y_true.shape)*ADD

    perf_1.set_current_value(y_true=y_true, y_pred=y_pred)

    perf_2 = Performance(metric_function=PERF_PARAMS['metric_function'],
                         abreviation=PERF_PARAMS['abreviation'],
                         min_value=PERF_PARAMS['min_value'],
                         max_value=PERF_PARAMS['max_value'],
                         is_bigger_better=PERF_PARAMS['is_bigger_better'],
                         version_length=VERSION_LENGTH)

    y_pred = y_true + np.ones(shape=y_true.shape)*(ADD + 5.0)

    perf_2.set_current_value(y_true=y_true, y_pred=y_pred)

    assert (perf_1 != perf_2)


@pytest.mark.parametrize("VERSION_LENGTH", VERSION_LENGTH_LIST)
@pytest.mark.parametrize("PERF_PARAMS", PERF_PARAMS_LIST)
@pytest.mark.parametrize("RANDOM_SIZE", RANDOM_SIZE_LIST)
def test_morethan_operator(PERF_PARAMS, VERSION_LENGTH, RANDOM_SIZE):
    perf_1 = Performance(metric_function=PERF_PARAMS['metric_function'],
                         abreviation=PERF_PARAMS['abreviation'],
                         min_value=PERF_PARAMS['min_value'],
                         max_value=PERF_PARAMS['max_value'],
                         is_bigger_better=PERF_PARAMS['is_bigger_better'],
                         version_length=VERSION_LENGTH)
    ADD = 1.00
    y_true = np.random.normal(size=(RANDOM_SIZE, 1))
    y_pred = y_true + np.ones(shape=y_true.shape)*ADD

    perf_1.set_current_value(y_true=y_true, y_pred=y_pred)

    perf_2 = Performance(metric_function=PERF_PARAMS['metric_function'],
                         abreviation=PERF_PARAMS['abreviation'],
                         min_value=PERF_PARAMS['min_value'],
                         max_value=PERF_PARAMS['max_value'],
                         is_bigger_better=PERF_PARAMS['is_bigger_better'],
                         version_length=VERSION_LENGTH)

    y_pred = y_true + np.ones(shape=y_true.shape)*(ADD + 15.0)

    perf_2.set_current_value(y_true=y_true, y_pred=y_pred)

    assert (perf_1 > perf_2)


@pytest.mark.parametrize("VERSION_LENGTH", VERSION_LENGTH_LIST)
@pytest.mark.parametrize("PERF_PARAMS", PERF_PARAMS_LIST)
@pytest.mark.parametrize("RANDOM_SIZE", RANDOM_SIZE_LIST)
def test_morethanorequal_operator(PERF_PARAMS, VERSION_LENGTH, RANDOM_SIZE):
    perf_1 = Performance(metric_function=PERF_PARAMS['metric_function'],
                         abreviation=PERF_PARAMS['abreviation'],
                         min_value=PERF_PARAMS['min_value'],
                         max_value=PERF_PARAMS['max_value'],
                         is_bigger_better=PERF_PARAMS['is_bigger_better'],
                         version_length=VERSION_LENGTH)
    ADD = 1.00
    y_true = np.random.normal(size=(RANDOM_SIZE, 1))
    y_pred = y_true + np.ones(shape=y_true.shape)*ADD

    perf_1.set_current_value(y_true=y_true, y_pred=y_pred)

    perf_2 = Performance(metric_function=PERF_PARAMS['metric_function'],
                         abreviation=PERF_PARAMS['abreviation'],
                         min_value=PERF_PARAMS['min_value'],
                         max_value=PERF_PARAMS['max_value'],
                         is_bigger_better=PERF_PARAMS['is_bigger_better'],
                         version_length=VERSION_LENGTH)

    y_pred = y_true + np.ones(shape=y_true.shape)*(ADD + 15.0)

    perf_2.set_current_value(y_true=y_true, y_pred=y_pred)

    assert (perf_1 >= perf_2)


@pytest.mark.parametrize("VERSION_LENGTH", VERSION_LENGTH_LIST)
@pytest.mark.parametrize("PERF_PARAMS", PERF_PARAMS_LIST)
@pytest.mark.parametrize("RANDOM_SIZE", RANDOM_SIZE_LIST)
def test_lessthan_operator(PERF_PARAMS, VERSION_LENGTH, RANDOM_SIZE):
    perf_1 = Performance(metric_function=PERF_PARAMS['metric_function'],
                         abreviation=PERF_PARAMS['abreviation'],
                         min_value=PERF_PARAMS['min_value'],
                         max_value=PERF_PARAMS['max_value'],
                         is_bigger_better=PERF_PARAMS['is_bigger_better'],
                         version_length=VERSION_LENGTH)
    ADD = 1.00
    y_true = np.random.normal(size=(RANDOM_SIZE, 1))
    y_pred = y_true + np.ones(shape=y_true.shape)*ADD

    perf_1.set_current_value(y_true=y_true, y_pred=y_pred)

    perf_2 = Performance(metric_function=PERF_PARAMS['metric_function'],
                         abreviation=PERF_PARAMS['abreviation'],
                         min_value=PERF_PARAMS['min_value'],
                         max_value=PERF_PARAMS['max_value'],
                         is_bigger_better=PERF_PARAMS['is_bigger_better'],
                         version_length=VERSION_LENGTH)

    y_pred = y_true + np.ones(shape=y_true.shape)*(ADD + 15.0)

    perf_2.set_current_value(y_true=y_true, y_pred=y_pred)

    assert (perf_2 < perf_1)


@pytest.mark.parametrize("VERSION_LENGTH", VERSION_LENGTH_LIST)
@pytest.mark.parametrize("PERF_PARAMS", PERF_PARAMS_LIST)
@pytest.mark.parametrize("RANDOM_SIZE", RANDOM_SIZE_LIST)
def test_lessthanorequal_operator(PERF_PARAMS, VERSION_LENGTH, RANDOM_SIZE):
    perf_1 = Performance(metric_function=PERF_PARAMS['metric_function'],
                         abreviation=PERF_PARAMS['abreviation'],
                         min_value=PERF_PARAMS['min_value'],
                         max_value=PERF_PARAMS['max_value'],
                         is_bigger_better=PERF_PARAMS['is_bigger_better'],
                         version_length=VERSION_LENGTH)
    ADD = 1.00
    y_true = np.random.normal(size=(RANDOM_SIZE, 1))
    y_pred = y_true + np.ones(shape=y_true.shape)*ADD

    perf_1.set_current_value(y_true=y_true, y_pred=y_pred)

    perf_2 = Performance(metric_function=PERF_PARAMS['metric_function'],
                         abreviation=PERF_PARAMS['abreviation'],
                         min_value=PERF_PARAMS['min_value'],
                         max_value=PERF_PARAMS['max_value'],
                         is_bigger_better=PERF_PARAMS['is_bigger_better'],
                         version_length=VERSION_LENGTH)

    y_pred = y_true + np.ones(shape=y_true.shape)*(ADD + 15.0)

    perf_2.set_current_value(y_true=y_true, y_pred=y_pred)

    assert (perf_2 <= perf_1)
